function DecisionTree(DatasetFolder)
    if(nargin<1)
        DatasetFolder='Kavraki'; % Use Ben dataset by default
    end
    
    if(strcmp(DatasetFolder,'GorkemUCM'))
        % Train the Logistic Regression with One against All methodology
        DecisionTree_Train_GorkemUCM(DatasetFolder);

        % Classify the data
        DecisionTree_Classify_GorkemUCM(DatasetFolder); 
    elseif(strcmp(DatasetFolder,'Kavraki'))
        % Train the Logistic Regression with One against All methodology
        DecisionTree_Train_Kavraki(DatasetFolder);

        % Classify the data
        DecisionTree_Classify_Kavraki(DatasetFolder); 
    end
end

function DecisionTree_Classify_GorkemUCM(DatasetFolder)
    for k=1:19
        j=k*5;
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');
            filename = strcat(DatasetFolder, '/DecisionTree/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'gdiT', 'twoingT', 'devianceT');
            % Note: find no significant difference between the 3 methods
            
            %-----------------------------------------------------
            % Splitting using GDI method
            %-----------------------------------------------------
            tic;
            yPredicted = eval(gdiT, classificationData);
            Timing(n,1)=toc;
            yfit=str2double(yPredicted);

            D=0.3048*sqrt(sum((cartesianLabels(yfit,:)-cartesianLabels(classificationLabels,:)).^2,2));
            Distance(n,:)=D';

            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/size(classificationLabels,1);
            end          
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(k).X=gdiErrorPlotX;
        errorPlot(k).Y=mean(gdiErrorPlotY);
        errorPlot(k).Distance=Distance;
        errorPlot(k).Timing=Timing;
        
%        figure
        % If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end
    end
    
    % Save the data to plot later on...
    %filename = strcat(DatasetFolder, '/DecisionTree/Results.mat');
    %save(filename, 'errorPlot');
end

function DecisionTree_Train_GorkemUCM(DatasetFolder)
    for n=1:50
        for i=5:5:95
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');
            
            % Train our classifier
            %B(c,:) = glmfit(trainingData, trainingLabels, 'binomial','link','logit')';

            tic;
            gdiT = classregtree(trainingData, trainingLabels, 'method','classification', 'prune', 'off', 'splitcriterion', 'gdi');
            gditTrainingTime=toc;
            
            
            tic;
            twoingT = classregtree(trainingData, trainingLabels, 'method','classification', 'prune', 'off', 'splitcriterion', 'twoing');
            twoingTrainingTime=toc;
            
            tic;
            devianceT = classregtree(trainingData, trainingLabels, 'method','classification', 'prune', 'off', 'splitcriterion', 'deviance');
            deviancetrainingTime=toc;
            
            filename = strcat(DatasetFolder, '/DecisionTree/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'gdiT', 'twoingT', 'devianceT', 'gditTrainingTime','twoingTrainingTime','deviancetrainingTime');
        end  
    end
end

function DecisionTree_Classify_Kavraki(DatasetFolder)
    for j=1:19
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');
            filename = strcat(DatasetFolder, '/DecisionTree/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'gdiT_F1', 'gdiT_F2', 'gdiT_F3', 'gditTrainingTime_F1', 'gditTrainingTime_F2', 'gditTrainingTime_F3');
            
            %--------------------- 1st Floor
            tic;
            yPredicted_F1 = eval(gdiT_F1, classificationData_F1);
            Timing(n,1)=toc;
            yfit_F1=str2double(yPredicted_F1);
            D_F1=sqrt(sum((cartesianLabels_F1(yfit_F1,:)-cartesianLabels_F1(classificationLabels_F1,:)).^2,2))/16.68;
            
            %--------------------- 2nd Floor
            tic;
            yPredicted_F2 = eval(gdiT_F2, classificationData_F2);
            Timing(n,2)=toc;
            yfit_F2=str2double(yPredicted_F2);
            D_F2=sqrt(sum((cartesianLabels_F2(yfit_F2,:)-cartesianLabels_F2(classificationLabels_F2,:)).^2,2))/16.68;
            
            %--------------------- 3rd Floor
            tic;
            yPredicted_F3 = eval(gdiT_F3, classificationData_F3);
            Timing(n,3)=toc;
            yfit_F3=str2double(yPredicted_F3);
            D_F3=sqrt(sum((cartesianLabels_F3(yfit_F3,:)-cartesianLabels_F3(classificationLabels_F3,:)).^2,2))/16.68;
            
            D=[D_F1;D_F2;D_F3];
            Distance(n,:)=D';

            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/(size(classificationLabels_F1,1)+size(classificationLabels_F2,1)+size(classificationLabels_F3,1));
            end          
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(j).X=gdiErrorPlotX;
        errorPlot(j).Y=mean(gdiErrorPlotY);
        errorPlot(j).Distance=Distance;
        errorPlot(j).Timing=Timing;
        
%         figure
%         %If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end
    end
    
    % Save the data to plot later on...
    filename = strcat(DatasetFolder, '/DecisionTree/Results.mat');
    save(filename, 'errorPlot');
end

function DecisionTree_Train_Kavraki(DatasetFolder)
    for n=1:50
        for i=1:1:19
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');

            tic;
            gdiT_F1 = classregtree(trainingData_F1, trainingLabels_F1, 'method','classification', 'prune', 'off', 'splitcriterion', 'gdi');
            gditTrainingTime_F1=toc;
            
            tic;
            gdiT_F2 = classregtree(trainingData_F2, trainingLabels_F2, 'method','classification', 'prune', 'off', 'splitcriterion', 'gdi');
            gditTrainingTime_F2=toc;
            
            tic;
            gdiT_F3 = classregtree(trainingData_F3, trainingLabels_F3, 'method','classification', 'prune', 'off', 'splitcriterion', 'gdi');
            gditTrainingTime_F3=toc;
            
            filename = strcat(DatasetFolder, '/DecisionTree/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'gdiT_F1', 'gdiT_F2', 'gdiT_F3', 'gditTrainingTime_F1', 'gditTrainingTime_F2', 'gditTrainingTime_F3');
        end  
    end
end

