function GaussianMixtureBen(DatasetFolder)
    if(nargin<1)
        DatasetFolder='GorkemUCM'; % Use Ben dataset by default
    end
    
    if(strcmp(DatasetFolder,'GorkemUCM'))
        % Train the Logistic Regression with One against All methodology
        %GaussianMixtureBen_Train_GorkemUCM(DatasetFolder);

        % Classify the data
        GaussianMixtureBen_Classify_GorkemUCM(DatasetFolder);
    elseif(strcmp(DatasetFolder,'Kavraki'))
        % Train the Logistic Regression with One against All methodology
        GaussianMixtureBen_Train_Kavraki(DatasetFolder);

        % Classify the data
        GaussianMixtureBen_Classify_Kavraki(DatasetFolder); 
    end
end

function GaussianMixtureBen_Classify_GorkemUCM(DatasetFolder)
    for k=1:2
        j=k*5;
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:5
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');
            filename = strcat(DatasetFolder, '/GaussianMixtureBen/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'Gaussians', 'trainingTime');

            numLoc=size(cartesianLabels,1);
            params=size(trainingData,2);

            Mean=Gaussians(:,1:2:size(Gaussians,2));
            Mean=reshape(Mean', 1, size(Mean,1)*size(Mean,2))';
            Var=Gaussians(:,2:2:size(Gaussians,2));
            Var=reshape(Var', 1, size(Var,1)*size(Var,2))';
            
            tic;
            yfit = zeros(size(classificationData,1),size(Gaussians,1));
            for ex=1:size(classificationData,1) % Loop through all examples
                exToClassify=repmat(classificationData(ex,:),1,numLoc)';
                exResults = normpdf(exToClassify, Mean, Var);
                exResults(Var==0 & exToClassify==Mean)=1;
                exResults(Var==0 & exToClassify~=Mean)=0.00001;
                exResults(exResults<0.00001)=0.00001;
                exResults = reshape(exResults,params,numLoc)';
                yfit(ex,:)=prod(exResults,2)';
            end
            [R, I] = max(yfit');
            Timing(n,1)=toc;

            D=0.3048*sqrt(sum((cartesianLabels(I,:)-cartesianLabels(classificationLabels,:)).^2,2));
            Distance(n,:)=D';
            mean(D)
            
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/size(classificationLabels,1);
            end
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(k).X=gdiErrorPlotX;
        errorPlot(k).Y=mean(gdiErrorPlotY);
        errorPlot(k).Distance=Distance;
        errorPlot(k).Timing=Timing;
        
        figure
        % If we only have one dataset
        if(size(gdiErrorPlotY,1)==1)
            plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
        else
            plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
        end  
    end
    
    % Save the data to plot later on...
%     filename = strcat(DatasetFolder, '/GaussianMixtureBen/Results.mat');
%     save(filename, 'errorPlot');
    
end

function GaussianMixtureBen_Train_GorkemUCM(DatasetFolder)
    for n=1:50
        for i=5:5:95
            clear t*;
            clear c*;
            clear G*;
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');

            numLoc=size(cartesianLabels,1);
            params=size(trainingData,2);
            Gaussians=zeros(numLoc,params*2);
            
            tic;
            % Compute Gaussian for each location, and each AP
            for c=1:numLoc % Loop through each location
                for g=1:params % Loop through each access point
                    % Get the data for just the class/location we are interested in
                    classData = trainingData(trainingLabels==c,:);
                    
                    % Store gaussian's mean
                    Gaussians(c,2*(g-1)+1)=mean(classData(:,g));
                    
                    % Store gaussian's standard deviation
                    Gaussians(c,2*(g-1)+2)=std(classData(:,g));
                end
            end
            trainingTime=toc;

            filename = strcat(DatasetFolder, '/GaussianMixtureBen/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'Gaussians', 'trainingTime');
        end  
    end
end

function GaussianMixtureBen_Classify_Kavraki(DatasetFolder)
    for j=1:19
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');
            filename = strcat(DatasetFolder, '/GaussianMixtureBen/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'Gaussians_F1', 'Gaussians_F2', 'Gaussians_F3', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');

            numLoc_F1=size(cartesianLabels_F1,1);
            params_F1=size(trainingData_F1,2);

            Mean_F1=Gaussians_F1(:,1:2:size(Gaussians_F1,2));
            Mean_F1=reshape(Mean_F1', 1, size(Mean_F1,1)*size(Mean_F1,2))';
            Var_F1=Gaussians_F1(:,2:2:size(Gaussians_F1,2));
            Var_F1=reshape(Var_F1', 1, size(Var_F1,1)*size(Var_F1,2))';
            
            tic;
            yfit_F1 = zeros(size(classificationData_F1,1),size(Gaussians_F1,1));
            for ex=1:size(classificationData_F1,1) % Loop through all examples
                exToClassify_F1=repmat(classificationData_F1(ex,:),1,numLoc_F1)';
                exResults_F1=normpdf(exToClassify_F1, Mean_F1, Var_F1);
                exResults_F1(Var_F1==0 & exToClassify_F1==Mean_F1)=1;
                exResults_F1(Var_F1==0 & exToClassify_F1~=Mean_F1)=0.00001;
                exResults_F1(exResults_F1<0.00001)=0.00001;
                exResults_F1 = reshape(exResults_F1,params_F1,numLoc_F1)';
                yfit_F1(ex,:)=prod(exResults_F1,2)';
            end
            [R_F1, I_F1] = max(yfit_F1');
            Timing(n,1)=toc;
            D_F1=sqrt(sum((cartesianLabels_F1(I_F1,:)-cartesianLabels_F1(classificationLabels_F1,:)).^2,2))/16.68;
            
            
            
            numLoc_F2=size(cartesianLabels_F2,1);
            params_F2=size(trainingData_F2,2);

            Mean_F2=Gaussians_F2(:,1:2:size(Gaussians_F2,2));
            Mean_F2=reshape(Mean_F2', 1, size(Mean_F2,1)*size(Mean_F2,2))';
            Var_F2=Gaussians_F2(:,2:2:size(Gaussians_F2,2));
            Var_F2=reshape(Var_F2', 1, size(Var_F2,1)*size(Var_F2,2))';
            
            tic;
            yfit_F2 = zeros(size(classificationData_F2,1),size(Gaussians_F2,1));
            for ex=1:size(classificationData_F2,1) % Loop through all examples
                exToClassify_F2=repmat(classificationData_F2(ex,:),1,numLoc_F2)';
                exResults_F2=normpdf(exToClassify_F2, Mean_F2, Var_F2);
                exResults_F2(Var_F2==0 & exToClassify_F2==Mean_F2)=1;
                exResults_F2(Var_F2==0 & exToClassify_F2~=Mean_F2)=0.00001;
                exResults_F2(exResults_F2<0.00001)=0.00001;
                exResults_F2 = reshape(exResults_F2,params_F2,numLoc_F2)';
                yfit_F2(ex,:)=prod(exResults_F2,2)';
            end
            [R_F2, I_F2] = max(yfit_F2');
            Timing(n,2)=toc;
            D_F2=sqrt(sum((cartesianLabels_F2(I_F2,:)-cartesianLabels_F2(classificationLabels_F2,:)).^2,2))/16.68;
            
            
            
            numLoc_F3=size(cartesianLabels_F3,1);
            params_F3=size(trainingData_F3,2);

            Mean_F3=Gaussians_F3(:,1:2:size(Gaussians_F3,2));
            Mean_F3=reshape(Mean_F3', 1, size(Mean_F3,1)*size(Mean_F3,2))';
            Var_F3=Gaussians_F3(:,2:2:size(Gaussians_F3,2));
            Var_F3=reshape(Var_F3', 1, size(Var_F3,1)*size(Var_F3,2))';
            
            tic;
            yfit_F3 = zeros(size(classificationData_F3,1),size(Gaussians_F3,1));
            for ex=1:size(classificationData_F3,1) % Loop through all examples
                exToClassify_F3=repmat(classificationData_F3(ex,:),1,numLoc_F3)';
                exResults_F3=normpdf(exToClassify_F3, Mean_F3, Var_F3);
                exResults_F3(Var_F3==0 & exToClassify_F3==Mean_F3)=1;
                exResults_F3(Var_F3==0 & exToClassify_F3~=Mean_F3)=0.00001;
                exResults_F3(exResults_F3<0.00001)=0.00001;
                exResults_F3 = reshape(exResults_F3,params_F3,numLoc_F3)';
                yfit_F3(ex,:)=prod(exResults_F3,2)';
            end
            [R_F3, I_F3] = max(yfit_F3');
            Timing(n,3)=toc;
            D_F3=sqrt(sum((cartesianLabels_F3(I_F3,:)-cartesianLabels_F3(classificationLabels_F3,:)).^2,2))/16.68;
            
            
            D=[D_F1;D_F2;D_F3];
            Distance(n,:)=D';
            
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/(size(classificationLabels_F1,1)+size(classificationLabels_F2,1)+size(classificationLabels_F3,1));
            end
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(j).X=gdiErrorPlotX;
        errorPlot(j).Y=mean(gdiErrorPlotY);
        errorPlot(j).Distance=Distance;
        errorPlot(j).Timing=Timing;
        
%         figure
%         % If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end  
    end
    
    % Save the data to plot later on...
    filename = strcat(DatasetFolder, '/GaussianMixtureBen/Results.mat');
    save(filename, 'errorPlot');
    
end

function GaussianMixtureBen_Train_Kavraki(DatasetFolder)
    for n=1:50
        for i=1:1:19
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');

            numLoc_F1=size(cartesianLabels_F1,1);
            params_F1=size(trainingData_F1,2);
            Gaussians_F1=zeros(numLoc_F1,params_F1*2);
            
            numLoc_F2=size(cartesianLabels_F2,1);
            params_F2=size(trainingData_F2,2);
            Gaussians_F2=zeros(numLoc_F2,params_F2*2);
            
            numLoc_F3=size(cartesianLabels_F3,1);
            params_F3=size(trainingData_F3,2);
            Gaussians_F3=zeros(numLoc_F3,params_F3*2);
            
            tic;
            % Compute Gaussian for each location, and each AP
            for c=1:numLoc_F1 % Loop through each location
                for g=1:params_F1 % Loop through each access point
                    % Get the data for just the class/location we are interested in
                    classData_F1 = trainingData_F1(trainingLabels_F1==c,:);
                    
                    % Store gaussian's mean
                    Gaussians_F1(c,2*(g-1)+1)=mean(classData_F1(:,g));
                    
                    % Store gaussian's standard deviation
                    Gaussians_F1(c,2*(g-1)+2)=std(classData_F1(:,g));
                end
            end
            trainingTime_F1=toc;
            
            tic;
            % Compute Gaussian for each location, and each AP
            for c=1:numLoc_F2 % Loop through each location
                for g=1:params_F2 % Loop through each access point
                    % Get the data for just the class/location we are interested in
                    classData_F2 = trainingData_F2(trainingLabels_F2==c,:);
                    
                    % Store gaussian's mean
                    Gaussians_F2(c,2*(g-1)+1)=mean(classData_F2(:,g));
                    
                    % Store gaussian's standard deviation
                    Gaussians_F2(c,2*(g-1)+2)=std(classData_F2(:,g));
                end
            end
            trainingTime_F2=toc;
            
            tic;
            % Compute Gaussian for each location, and each AP
            for c=1:numLoc_F3 % Loop through each location
                for g=1:params_F3 % Loop through each access point
                    % Get the data for just the class/location we are interested in
                    classData_F3 = trainingData_F3(trainingLabels_F3==c,:);
                    
                    % Store gaussian's mean
                    Gaussians_F3(c,2*(g-1)+1)=mean(classData_F3(:,g));
                    
                    % Store gaussian's standard deviation
                    Gaussians_F3(c,2*(g-1)+2)=std(classData_F3(:,g));
                end
            end
            trainingTime_F3=toc;

            filename = strcat(DatasetFolder, '/GaussianMixtureBen/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'Gaussians_F1', 'Gaussians_F2', 'Gaussians_F3', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');
        end  
    end
end

