function LRoneVall(DatasetFolder)
    if(nargin<1)
        DatasetFolder='Kavraki'; % Use Ben dataset by default
    end
    
    if(strcmp(DatasetFolder,'GorkemUCM'))
        % Train the Logistic Regression with One against All methodology
        LRoneVall_Train_GorkemUCM(DatasetFolder);

        % Classify the data
        LRoneVall_Classify_GorkemUCM(DatasetFolder);
    elseif(strcmp(DatasetFolder,'Kavraki'))
        % Train the Logistic Regression with One against All methodology
        LRoneVall_Train_Kavraki(DatasetFolder);

        % Classify the data
        LRoneVall_Classify_Kavraki(DatasetFolder); 
    end
    
end

function LRoneVall_Classify_GorkemUCM(DatasetFolder)
    for k=1:19
        j=k*5;
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');
            filename = strcat(DatasetFolder, '/LRoneVall/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'B');

            yfit=[];
            numLoc=size(cartesianLabels,1);
            tic;
            for c=1:numLoc
                yfit(:,c) = glmval(B(c,:)',classificationData,'logit');
            end
            [R, I] = max(yfit');
            Timing(n,1)=toc;  

            D=0.3048*sqrt(sum((cartesianLabels(I,:)-cartesianLabels(classificationLabels,:)).^2,2));
            Distance(n,:)=D';

            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/size(classificationLabels,1);
            end          
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(k).X=gdiErrorPlotX;
        errorPlot(k).Y=mean(gdiErrorPlotY);
        errorPlot(k).Distance=Distance;
        errorPlot(k).Timing=Timing;
        
%        figure   
        % If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end
    end
    
    % Save the data to plot later on...
    %filename = strcat(DatasetFolder, '/LRoneVall/Results.mat');
    %save(filename, 'errorPlot');
end

function LRoneVall_Train_GorkemUCM(DatasetFolder)
    for n=49:50
        for i=5:5:95
            clear t*;
            clear c*;
            clear B;
            
            tic;
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');

            numLoc=size(cartesianLabels,1);
            params=size(trainingData,2);
            B = zeros(numLoc,params+1);

            % Need to have one classigfier for each class
            %   NOTE: we have numLoc classes
            for c=1:numLoc 
                newTrainingLabels=(trainingLabels==c);
                % Train our classifier
                B(c,:) = glmfit(trainingData, newTrainingLabels, 'binomial','link','logit')';
            end
            trainingTime=toc;

            filename = strcat(DatasetFolder, '/LRoneVall/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'B', 'trainingTime');
        end  
    end
end

function LRoneVall_Classify_Kavraki(DatasetFolder)
    for j=1:19
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');
            filename = strcat(DatasetFolder, '/LRoneVall/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'B_F1', 'B_F2', 'B_F3', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');

            yfit_F1=[];
            numLoc_F1=size(cartesianLabels_F1,1);
            tic;
            for c=1:numLoc_F1
                yfit_F1(:,c) = glmval(B_F1(c,:)',classificationData_F1,'logit');
            end
            [R_F1, I_F1] = max(yfit_F1');
            Timing(n,1)=toc;  

            D_F1=sqrt(sum((cartesianLabels_F1(I_F1,:)-cartesianLabels_F1(classificationLabels_F1,:)).^2,2))/16.68;
            
            
            yfit_F2=[];
            numLoc_F2=size(cartesianLabels_F2,1);
            tic;
            for c=1:numLoc_F2
                yfit_F2(:,c) = glmval(B_F2(c,:)',classificationData_F2,'logit');
            end
            [R_F2, I_F2] = max(yfit_F2');
            Timing(n,2)=toc;  

            D_F2=sqrt(sum((cartesianLabels_F2(I_F2,:)-cartesianLabels_F2(classificationLabels_F2,:)).^2,2))/16.68;
            
            
            yfit_F3=[];
            numLoc_F3=size(cartesianLabels_F3,1);
            tic;
            for c=1:numLoc_F3
                yfit_F3(:,c) = glmval(B_F3(c,:)',classificationData_F3,'logit');
            end
            [R_F3, I_F3] = max(yfit_F3');
            Timing(n,3)=toc;  

            D_F3=sqrt(sum((cartesianLabels_F3(I_F3,:)-cartesianLabels_F3(classificationLabels_F3,:)).^2,2))/16.68;
            
            
            D=[D_F1;D_F2;D_F3];
            Distance(n,:)=D';

            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/(size(classificationLabels_F1,1)+size(classificationLabels_F2,1)+size(classificationLabels_F3,1));
            end          
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(j).X=gdiErrorPlotX;
        errorPlot(j).Y=mean(gdiErrorPlotY);
        errorPlot(j).Distance=Distance;
        errorPlot(j).Timing=Timing;
        
%         figure   
%         %If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end
    end
    
    % Save the data to plot later on...
    filename = strcat(DatasetFolder, '/LRoneVall/Results.mat');
    save(filename, 'errorPlot');
end

function LRoneVall_Train_Kavraki(DatasetFolder)
    for n=1:50
        for i=1:1:19
            n
            tic;
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');

            
            numLoc_F1=size(cartesianLabels_F1,1);
            params_F1=size(trainingData_F1,2);
            B_F1 = zeros(numLoc_F1,params_F1+1);

            % Need to have one classigfier for each class
            %   NOTE: we have numLoc classes
            for c=1:numLoc_F1 
                newTrainingLabels_F1=(trainingLabels_F1==c);
                % Train our classifier
                B_F1(c,:) = glmfit(trainingData_F1, newTrainingLabels_F1, 'binomial','link','logit')';
            end
            trainingTime_F1=toc;

            
            numLoc_F2=size(cartesianLabels_F2,1);
            params_F2=size(trainingData_F2,2);
            B_F2 = zeros(numLoc_F2,params_F2+1);

            % Need to have one classigfier for each class
            %   NOTE: we have numLoc classes
            for c=1:numLoc_F2 
                newTrainingLabels_F2=(trainingLabels_F2==c);
                % Train our classifier
                B_F2(c,:) = glmfit(trainingData_F2, newTrainingLabels_F2, 'binomial','link','logit')';
            end
            trainingTime_F2=toc;
            
            
            numLoc_F3=size(cartesianLabels_F3,1);
            params_F3=size(trainingData_F3,2);
            B_F3 = zeros(numLoc_F3,params_F3+1);

            % Need to have one classigfier for each class
            %   NOTE: we have numLoc classes
            for c=1:numLoc_F3 
                newTrainingLabels_F3=(trainingLabels_F3==c);
                % Train our classifier
                B_F3(c,:) = glmfit(trainingData_F3, newTrainingLabels_F3, 'binomial','link','logit')';
            end
            trainingTime_F3=toc;
            
            filename = strcat(DatasetFolder, '/LRoneVall/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'B_F1', 'B_F2', 'B_F3', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');
        end  
    end
end

