function MultinomialLogit(DatasetFolder)
    if(nargin<1)
        DatasetFolder='Kavraki'; % Use Ben dataset by default
    end
    
    if(strcmp(DatasetFolder,'GorkemUCM'))
        % Train the Logistic Regression with One against All methodology
        MultinomialLogit_Train_GorkemUCM(DatasetFolder);

        % Classify the data
        MultinomialLogit_Classify_GorkemUCM(DatasetFolder);
    elseif(strcmp(DatasetFolder,'Kavraki'))
        % Train the Logistic Regression with One against All methodology
        MultinomialLogit_Train_Kavraki(DatasetFolder);

        % Classify the data
        MultinomialLogit_Classify_Kavraki(DatasetFolder); 
    end
    
end

function MultinomialLogit_Classify_GorkemUCM(DatasetFolder)
    for k=1:19
        j=k*5;
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');
            filename = strcat(DatasetFolder, '/MultinomialLogit/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'B');

            tic;
            yfit = mnrval(B,classificationData,'model','hierarchical');
            [R, I] = max(yfit');
            Timing(n,1)=toc;

            D=0.3048*sqrt(sum((cartesianLabels(I,:)-cartesianLabels(classificationLabels,:)).^2,2));
            Distance(n,:)=D';

            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/size(classificationLabels,1);
            end          
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(k).X=gdiErrorPlotX;
        errorPlot(k).Y=mean(gdiErrorPlotY);
        errorPlot(k).Distance=Distance;
        errorPlot(k).Timing=Timing;
        
%        figure   
        % If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end
    end
    
    % Save the data to plot later on...
    %filename = strcat(DatasetFolder, '/MultinomialLogit/Results.mat');
    %save(filename, 'errorPlot');
    
end

function MultinomialLogit_Train_GorkemUCM(DatasetFolder)
    for n=1:50
        for i=5:5:95
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');

            tic;
            B = mnrfit(trainingData, trainingLabels, 'model', 'hierarchical');
            trainingTime=toc;
            
            filename = strcat(DatasetFolder, '/MultinomialLogit/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'B', 'trainingTime');
        end  
    end
end

function MultinomialLogit_Classify_Kavraki(DatasetFolder)
    for j=1:19
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');
            filename = strcat(DatasetFolder, '/MultinomialLogit/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'B_F1', 'B_F2', 'B_F3', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');

            tic;
            yfit_F1 = mnrval(B_F1,classificationData_F1,'model','hierarchical');
            [R_F1, I_F1] = max(yfit_F1');
            Timing(n,1)=toc;
            D_F1=sqrt(sum((cartesianLabels_F1(I_F1,:)-cartesianLabels_F1(classificationLabels_F1,:)).^2,2))/16.68;
            
            tic;
            yfit_F2 = mnrval(B_F2,classificationData_F2,'model','hierarchical');
            [R_F2, I_F2] = max(yfit_F2');
            Timing(n,2)=toc;
            D_F2=sqrt(sum((cartesianLabels_F2(I_F2,:)-cartesianLabels_F2(classificationLabels_F2,:)).^2,2))/16.68;
            
            tic;
            yfit_F3 = mnrval(B_F3,classificationData_F3,'model','hierarchical');
            [R_F3, I_F3] = max(yfit_F3');
            Timing(n,3)=toc;
            D_F3=sqrt(sum((cartesianLabels_F3(I_F3,:)-cartesianLabels_F3(classificationLabels_F3,:)).^2,2))/16.68;  
            
            D=[D_F1;D_F2;D_F3];
            Distance(n,:)=D';

            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/(size(classificationLabels_F1,1)+size(classificationLabels_F2,1)+size(classificationLabels_F3,1));
            end          
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(j).X=gdiErrorPlotX;
        errorPlot(j).Y=mean(gdiErrorPlotY);
        errorPlot(j).Distance=Distance;
        errorPlot(j).Timing=Timing;
        
%         figure   
%         %If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end
    end
    
    % Save the data to plot later on...
    filename = strcat(DatasetFolder, '/MultinomialLogit/Results.mat');
    save(filename, 'errorPlot');
    
end

function MultinomialLogit_Train_Kavraki(DatasetFolder)
    for n=1:50
        for i=1:19
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');

            tic;
            B_F1 = mnrfit(trainingData_F1, trainingLabels_F1, 'model', 'hierarchical');
            trainingTime_F1=toc;
            
            tic;
            B_F2 = mnrfit(trainingData_F2, trainingLabels_F2, 'model', 'hierarchical');
            trainingTime_F2=toc;
            
            tic;
            B_F3 = mnrfit(trainingData_F3, trainingLabels_F3, 'model', 'hierarchical');
            trainingTime_F3=toc;
            
            filename = strcat(DatasetFolder, '/MultinomialLogit/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'B_F1', 'B_F2', 'B_F3', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');
        end  
    end
end

