function NearestNeighbor(DatasetFolder)
    if(nargin<1)
        DatasetFolder='Kavraki'; % Use Ben dataset by default
    end
    
    if(strcmp(DatasetFolder,'GorkemUCM'))
        % Classify the data
        NearestNeighbor_Classify_GorkemUCM(DatasetFolder);
    elseif(strcmp(DatasetFolder,'Kavraki'))
        % Classify the data
        NearestNeighbor_Classify_Kavraki(DatasetFolder); 
    end
end

function NearestNeighbor_Classify_GorkemUCM(DatasetFolder)
    for k=1:19
        j=k*5;
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');

            tic;
            yfit = knnclassify(classificationData(1,:), trainingData, trainingLabels, 1);
            Timing(n,1)=toc

            D=0.3048*sqrt(sum((cartesianLabels(yfit,:)-cartesianLabels(classificationLabels,:)).^2,2));
            Distance(n,:)=D';
            mean(D);
            
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/size(classificationLabels,1);
            end
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(k).X=gdiErrorPlotX;
        errorPlot(k).Y=mean(gdiErrorPlotY);
        errorPlot(k).Distance=Distance;
        errorPlot(k).Timing=Timing;
        
%         figure
%         % If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end  
    end
    
    % Save the data to plot later on...
    filename = strcat(DatasetFolder, '/NearestNeighbor/Results.mat');
    save(filename, 'errorPlot');
    
end

function NearestNeighbor_Classify_Kavraki(DatasetFolder)
    for j=3:3
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');
            
            tic;
            yfit_F1 = knnclassify(classificationData_F1, trainingData_F1, trainingLabels_F1, 1);
            Timing(n,1)=toc;
            D_F1=sqrt(sum((cartesianLabels_F1(yfit_F1,:)-cartesianLabels_F1(classificationLabels_F1,:)).^2,2))/16.68;
            
            tic;
            yfit_F2 = knnclassify(classificationData_F2, trainingData_F2, trainingLabels_F2, 1);
            Timing(n,2)=toc;
            D_F2=sqrt(sum((cartesianLabels_F2(yfit_F2,:)-cartesianLabels_F2(classificationLabels_F2,:)).^2,2))/16.68;
            
            tic;
            yfit_F3 = knnclassify(classificationData_F3, trainingData_F3, trainingLabels_F3, 1);
            Timing(n,3)=toc;
            D_F3=sqrt(sum((cartesianLabels_F3(yfit_F3,:)-cartesianLabels_F3(classificationLabels_F3,:)).^2,2))/16.68;
            
            D=[D_F1;D_F2;D_F3];
            Distance(n,:)=D';
            mean(D);
            
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/(size(classificationLabels_F1,1)+size(classificationLabels_F2,1)+size(classificationLabels_F3,1));
            end
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(j).X=gdiErrorPlotX;
        errorPlot(j).Y=mean(gdiErrorPlotY);
        errorPlot(j).Distance=Distance;
        errorPlot(j).Timing=Timing;
        
%         figure
%         % If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end  
    end
    
    % Save the data to plot later on...
    filename = strcat(DatasetFolder, '/NearestNeighbor/Results.mat');
    save(filename, 'errorPlot');
    
end

