function ParticleFilter()
    runNumber = 1; % Run number of real robot, between 1 and 10
    trainingNumber = 1; % training number, between 1 and 50
    trainingPercent = 15; % Percentage of the data used for training, between 5:5:95 
    numParticules=5000;
    showDisplay=0;
    XtSaved=[];

    wifiCounter=1;
    filterCounter=1;
    
    ut=zeros(2,3);
    odomData=[];
    zt=[];
    
    filename = strcat('GorkemUCM/TrainingRandomized/Trained_', num2str(trainingPercent,'%02d'), '_', num2str(trainingNumber,'%02d'), '.mat');
    load(filename, 'cartesianLabels');
    filename = strcat('GorkemUCM/RandomForest/trained_', num2str(trainingPercent,'%02d'), '_', num2str(trainingNumber,'%02d'), '.mat');
    load(filename, 'rForest');
    
    % Setup the visual process
    if(showDisplay)
        colorimg=imread('map3.png');
        colorimg(:,:,1)=flipud(colorimg(:,:,1));
        colorimg(:,:,2)=flipud(colorimg(:,:,2));
        colorimg(:,:,3)=flipud(colorimg(:,:,3));
        fig=figure;
        %Origin=[92.0794710360741-1.50229403744196 2200-(1081.67115030551+10.8412906953541)];
        %surf([0-Origin(1) 1235-Origin(1)]*0.0656167979,[0-Origin(2) 2200-Origin(2)]*0.0656167979,repmat(-0.0001, [2 2]), colorimg,'facecolor','texture')
        %hold on;
        %plot1=scatter(0,0,1,0);
        %plot2=plot(0,0,'xr');
        axis equal;
        view([0 89]);
    end

    % Get wifi data for the appropriate run
    load(strcat('RealRobot/Runs/',num2str(runNumber,'%02d'),'.mat'), 'classificationData');
    wifiData=classificationData;
    
    % Get time stamps for wifi readings
    load('RealRobot_time.mat', 'wifiTimeStamps');
    
    % Start reading odometry file
    fid = fopen('RealRobot/dOdometry.txt','r');
    count=0;    
    while(~feof(fid) && wifiCounter<=size(wifiTimeStamps,1))
        s = fgets(fid); % Read one line
        oldOdomData = odomData;
        odomData = sscanf(s, '%f\t%f\t%f\t%f\t%f\t%f\t%d');
        filterCounter
        
        tic
        
        % If we have found a matching time between wifi and odo for the
        % first time, we initiate the particles (bootstrap algorithm
        if(wifiCounter==1 && odomData(1)==wifiTimeStamps(wifiCounter))
            zt=wifiData(wifiCounter,:);
            Xt = initializeParticles('RandomForest', wifiData(wifiCounter,:), cartesianLabels, rForest, numParticules);
%             XtSaved(size(XtSaved,1)+1,1).Xt=Xt;
            
            if(showDisplay)
                plot1=scatter(Xt(filterCounter).state(:,1)*3.28083,Xt(filterCounter).state(:,2)*3.28083,1,Xt(filterCounter).weight./sum(Xt(filterCounter).weight));
            end
            filterCounter=filterCounter+1;
            wifiCounter=wifiCounter+1;
        % If we only have an odometry reading
        else
            % If we have Odometry AND Wifi Readings at the same time
            if(odomData(1)==wifiTimeStamps(wifiCounter))
                zt=wifiData(wifiCounter,:);
                wifiCounter=wifiCounter+1;
            else
                zt=[]; % If we only have odometry, set measurement to null
            end
            
            % Make sure particles have been initialized
            if(filterCounter>1)
                % Create ut, which is comprised of xt and xt-1 from the robot's odometry
                ut(1,:)=oldOdomData(2:4)';
                ut(2,:)=odomData(2:4)';
    
                Xt = MCL(Xt, ut, zt, numParticules, 'RandomForest', cartesianLabels, rForest);
                
                if(showDisplay)
                    colormap(jet);
                    %plot(Xt.state(:,1)*3.28083, Xt.state(:,2)*3.28083, 'xk');
                    %set(plot1, 'XData', Xt(filterCounter).state(:,1)*3.28083, 'YData', Xt(filterCounter).state(:,2)*3.28083, 'CData', Xt(filterCounter).weight./sum(Xt(filterCounter).weight));
                    plot1=scatter(Xt(filterCounter).state(:,1)*3.28083,Xt(filterCounter).state(:,2)*3.28083,1,Xt(filterCounter).weight./sum(Xt(filterCounter).weight));
                    pause(0.01);
                end
                
%                 if(size(zt,1)~=0)
%                     XtSaved(size(XtSaved,1)+1,1).Xt=Xt;
%                 end
                
                filterCounter=filterCounter+1;
            end
        end
        
        if(size(zt,1)~=0 & filterCounter>1)
            updateTimeMeasurement(wifiCounter-1)=toc;
        elseif(filterCounter>1)
            updateTimeMotion(filterCounter-(wifiCounter))=toc;
        end
    end
    fclose(fid);
    
    save('ParticleFilter/timing.mat', 'updateTimeMeasurement', 'updateTimeMotion');
    %save('ParticleFilter/test3.mat', 'XtSaved');
    
    
    %Xt(timeStep) = MCL(Xt(timeStep-1), ut, zt, numParticles); 
end

function Xt = MCL(Xtm1, ut, zt, numParticules, modelname, cartesianLabels, rForest)
    
    Xtbar.state=zeros(numParticules,3);
    Xtbar.weight=zeros(numParticules,1);
    
%TO DO: make it faster by removing FOR loop
%     for m=1:numParticules
%         Xtbar.state(m,:) = sample_motion_model2(ut, Xtm1.state(m,:));
%     end
    Xtbar.state = sample_motion_model(ut, Xtm1.state);

    % We only resample if we have a measurement
    if(size(zt,1)~=0)
        Xtbar.weight = measurement_model(zt, Xtbar.state, modelname, cartesianLabels, rForest);
        
        if(sum(Xtbar.weight)==0)
        	'Error Coming-1'
        end
        
        % Normalize the weights
        Xtbar.weight = Xtbar.weight./sum(Xtbar.weight);

%         sumw=sum(Xtbar.weight);
%         if ~(sumw > 0)
%             'Error Coming-2'
%         end
        
        % The number of particles we want to resample using measurement
%         numDegrees=72; % Sample uniformaly around the 360 degree circle 72 times (5 degree increments)
%         measurementResampleParticles=0;
%         measurementResample=numDegrees*measurementResampleParticles;
%         
%         % The number of particles we want to resample to add 
%         thetaResampleParticles=0;
%         thetaResample=numDegrees*measurementResampleParticles;
        
        % The number of particles we want to resample using weights
        weightResample=numParticules;%-measurementResample-thetaResample;


        % Draw from the particules using the weights    
        %I = randsample(1:size(Xtbar.state,1),size(Xtbar.state,1),true,Xtbar.weight);
        I = randsample(1:size(Xtbar.state,1),weightResample,true,Xtbar.weight);
        
        % Add drawn particles to final results
        Xt.state(1:weightResample,:)=Xtbar.state(I,:);
        Xt.weight=Xtbar.weight(I);
    else
        Xt.state=Xtbar.state;
        Xt.weight=Xtm1.weight;
    end
    
end

function Xt = sample_motion_model(Ut, Xtm1)
    % Need to set the error parameters
    E1=0.025;
    E2=0.025;
    E3=0.025;
    E4=0.025;

    % Get the values from the parameters
    Xbar=Ut(1,1);
    Ybar=Ut(1,2);
    Tbar=wrapToPi(Ut(1,3));
    Xbarprime=Ut(2,1);
    Ybarprime=Ut(2,2);
    Tbarprime=wrapToPi(Ut(2,3));
    X = Xtm1(:,1);
    Y = Xtm1(:,2);
    T = wrapToPi(Xtm1(:,3));
    
    % Algorithm Starts Here - Taken from pg.136 of Trun's book
    rot1=wrapToPi(atan2(Ybarprime-Ybar, Xbarprime-Xbar)-Tbar);
    trans=sqrt((Xbar-Xbarprime)^2+(Ybar-Ybarprime)^2);
    rot2=wrapToPi(Tbarprime-Tbar-rot1);
    
    rot1Hat = wrapToPi(rot1 - normrnd(0, E1*abs(rot1)+E2*trans, [size(Xtm1,1) 1]));
    transHat = trans - normrnd(0, E3*trans+E4*abs(rot1)+abs(rot2), [size(Xtm1,1) 1]);
    rot2Hat = wrapToPi(rot2 - normrnd(0, E1*abs(rot2)+E2*trans, [size(Xtm1,1) 1]));
    
    Xprime = X+transHat.*cos(wrapToPi(T+rot1Hat));
    Yprime = Y+transHat.*sin(wrapToPi(T+rot1Hat));
    Tprime = wrapToPi(T + rot1Hat + rot2Hat);
    
    Xt = [Xprime Yprime Tprime];
end

function Xt = sample_motion_model2(Ut, Xtm1)
    % Need to set the error parameters
    E1=0.1;
    E2=0.1;
    E3=0.1;
    E4=0.1;

    % Get the values from the parameters
    Xbar=Ut(1,1);
    Ybar=Ut(1,2);
    Tbar=Ut(1,3);
    Xbarprime=Ut(2,1);
    Ybarprime=Ut(2,2);
    Tbarprime=Ut(2,3);
    X = Xtm1(1,1);
    Y = Xtm1(1,2);
    T = Xtm1(1,3);
    
    % Algorithm Starts Here - Taken from pg.136 of Trun's book
    rot1=atan2(Ybarprime-Ybar, Xbarprime-Xbar)-Tbar;
    trans=sqrt((Xbar-Xbarprime)^2+(Ybar-Ybarprime)^2);
    rot2=Tbarprime-Tbar-rot1;
    
    rot1Hat = rot1 - normrnd(0, E1*abs(rot1)+E2*trans);
    transHat = trans - normrnd(0, E3*trans+E4*(abs(rot1)+abs(rot2)));
    rot2Hat = rot2 - normrnd(0, E1*abs(rot2)+E2*trans);
    
    Xprime = X+transHat*cos(T+rot1Hat);
    Yprime = Y+transHat*sin(T+rot1Hat);
    Tprime = T + rot1Hat + rot2Hat;
    
    Xt = [Xprime Yprime Tprime];
end

function Wt = measurement_model(zt, Xt, modelname, cartesianLabels, rForest)
    if strcmp(modelname, 'RandomForest')
        GMM = getGMMfromRandomForest(zt, cartesianLabels, rForest);
        
        % Convert particles' state (in meters) to feet, before finding weight for them
        Xt(:,1:2)=Xt(:,1:2)*3.2808399;
        
        Wt = pdf(GMM, Xt(:,1:2));
    end
end

function Xt = initializeParticles(modelname, wifiMeasurement, cartesianLabels, rForest, numParticles)
    if strcmp(modelname, 'RandomForest')
        GMM = getGMMfromRandomForest(wifiMeasurement, cartesianLabels, rForest);
        
        % Draw M samples from the distribution
        [Y, I] = random(GMM,numParticles);
    
        % Convert the samples that are in feet to meters
        Y=Y.*0.3048;
        
        % Add drawn particles to final results
        Xt.state=[Y zeros(size(Y,1),1)];
        Xt.weight=pdf(GMM, Y);
    end
end

function GMM = getGMMfromRandomForest(wifiMeasurement, cartesianLabels, rForest)
    [yPredicted, scores] = predict(rForest, wifiMeasurement);

    % Set up the means of each Gaussian
    mu=cartesianLabels(scores(1,:)>0,:);
    
    % Setup the Sigma of each Gaussian
    sigma=[];
    for n=1:size(mu,1)
        sigma(:,:,n) = 5.5*eye(2);
    end
    
    % Setup the mixture proportions of each Gaussian
    p=scores(1,scores(1,:)>0);
    
    GMM = gmdistribution(mu,sigma,p);
end