function RandomForest(DatasetFolder)
    if(nargin<1)
        DatasetFolder='Kavraki'; % Use Ben dataset by default
    end
    
    if(strcmp(DatasetFolder,'GorkemUCM'))
        % Train the Logistic Regression with One against All methodology
        RandomForest_Train_GorkemUCM(DatasetFolder);

        % Classify the data
        RandomForest_Classify_GorkemUCM(DatasetFolder);
    elseif(strcmp(DatasetFolder,'Kavraki'))
        % Train the Logistic Regression with One against All methodology
        RandomForest_Train_Kavraki(DatasetFolder);

        % Classify the data
        RandomForest_Classify_Kavraki(DatasetFolder); 
    end
    
end

function RandomForest_Classify_GorkemUCM(DatasetFolder)
    for k=1:19
        j=k*5;
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');
            filename = strcat(DatasetFolder, '/RandomForest/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'rForest');
            
            tic;
            [yPredicted, scores] = predict(rForest, classificationData(1,:));
            Timing(n,1)=toc
            yfit=str2double(yPredicted);
            
            D=0.3048*sqrt(sum((cartesianLabels(yfit,:)-cartesianLabels(classificationLabels,:)).^2,2));
            Distance(n,:)=D';
            
            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/size(classificationLabels,1);
            end          
        end
    
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(k).X=gdiErrorPlotX;
        errorPlot(k).Y=mean(gdiErrorPlotY);
        errorPlot(k).Distance=Distance;
        errorPlot(k).Timing=Timing;
        
%        figure   
        % If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end
    end
    
    % Save the data to plot later on...
    %filename = strcat(DatasetFolder, '/RandomForest/Results.mat');
    %save(filename, 'errorPlot');
    
end

function RandomForest_Train_GorkemUCM(DatasetFolder)
    for n=1:50
        for i=5:5:95
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');

            tic;
            rForest = TreeBagger(50, trainingData, trainingLabels, 'method','classification');
            trainingTime=toc

            filename = strcat(DatasetFolder, '/RandomForest/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'rForest', 'trainingTime');
        end  
    end
end

function RandomForest_Classify_Kavraki(DatasetFolder)
    for j=1:19
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');
            filename = strcat(DatasetFolder, '/RandomForest/trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'rForest_F1', 'rForest_F2', 'rForest_F3', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');
            
            tic;
            [yPredicted_F1, scores_F1] = predict(rForest_F1, classificationData_F1);
            Timing(n,1)=toc;
            yfit_F1=str2double(yPredicted_F1);
            D_F1=sqrt(sum((cartesianLabels_F1(yfit_F1,:)-cartesianLabels_F1(classificationLabels_F1,:)).^2,2))/16.68;
            
            tic;
            [yPredicted_F2, scores_F2] = predict(rForest_F2, classificationData_F2);
            Timing(n,2)=toc;
            yfit_F2=str2double(yPredicted_F2);
            D_F2=sqrt(sum((cartesianLabels_F2(yfit_F2,:)-cartesianLabels_F2(classificationLabels_F2,:)).^2,2))/16.68;
            
            tic;
            [yPredicted_F3, scores_F3] = predict(rForest_F3, classificationData_F3);
            Timing(n,3)=toc;
            yfit_F3=str2double(yPredicted_F3);
            D_F3=sqrt(sum((cartesianLabels_F3(yfit_F3,:)-cartesianLabels_F3(classificationLabels_F3,:)).^2,2))/16.68;
            
            D=[D_F1;D_F2;D_F3];
            Distance(n,:)=D';
            
            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/(size(classificationLabels_F1,1)+size(classificationLabels_F2,1)+size(classificationLabels_F3,1));
            end          
        end
    
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(j).X=gdiErrorPlotX;
        errorPlot(j).Y=mean(gdiErrorPlotY);
        errorPlot(j).Distance=Distance;
        errorPlot(j).Timing=Timing;
        
%         figure   
%         %If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end
    end
    
    % Save the data to plot later on...
    filename = strcat(DatasetFolder, '/RandomForest/Results.mat');
    save(filename, 'errorPlot');
    
end

function RandomForest_Train_Kavraki(DatasetFolder)
    for n=1:50
        for i=1:19
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');

            tic;
            rForest_F1 = TreeBagger(50, trainingData_F1, trainingLabels_F1, 'method','classification');
            trainingTime_F1=toc;
            
            tic;
            rForest_F2 = TreeBagger(50, trainingData_F2, trainingLabels_F2, 'method','classification');
            trainingTime_F2=toc;
            
            tic;
            rForest_F3 = TreeBagger(50, trainingData_F3, trainingLabels_F3, 'method','classification');
            trainingTime_F3=toc;

            filename = strcat(DatasetFolder, '/RandomForest/trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'rForest_F1', 'rForest_F2', 'rForest_F3', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');
        end  
    end
end

