function RandomizeTraining_Gorkem(onlyUCM)
    load('Gorkem_data.mat');
    
    % If nothing was passed as a parameter, we only use UCM networks
    if(nargin<1)
        onlyUCM = 1;
    end
    
    % Where we will originally store the processed data
    cartesianLabels=[];
    Data=[];
    MacIndices=[];
    
    % Get unique indexes for the MAC addresses
    for i=1:size(data,1)
        
        % Check to see if we already have an entry for this cartesian label
        % If not, add it to the list of cartesian labels
        if(size(cartesianLabels,1)==0 || sum(cartesianLabels(:,1)==data(i).x*3&cartesianLabels(:,2)==data(i).y*3)==0)
            cartesianLabels(size(cartesianLabels,1)+1,1) = data(i).x*3; % *3 to make sure they are in feet
            cartesianLabels(size(cartesianLabels,1),2) = data(i).y*3;   % *3 to make sure they are in feet
        end
        
        for j=1:size(data(i,1).ap,1)
            
            % Only get UCM networks
            if(onlyUCM)
                if(strcmp(data(i,1).ap(j,1).essid, 'UCM-Guest"') || strcmp(data(i,1).ap(j,1).essid, 'UCMerced"'))
                    if(size(strmatch(data(i,1).ap(j,1).mac, MacIndices, 'exact'),1)==0) % If MAC is not already in the list
                        MacIndices(size(MacIndices,1)+1,:)=data(i,1).ap(j,1).mac;
                    end
                    
                    Data(i, strmatch(data(i,1).ap(j,1).mac, MacIndices, 'exact')) = data(i,1).ap(j,1).signal;
                end
            else
                if(size(strmatch(data(i,1).ap(j,1).mac, MacIndices, 'exact'),1)==0) % If MAC is not already in the list
                    MacIndices(size(MacIndices,1)+1,:)=data(i,1).ap(j,1).mac;
                end

                Data(i, strmatch(data(i,1).ap(j,1).mac, MacIndices, 'exact')) = data(i,1).ap(j,1).signal;         
            end
        end
    end
    
    % Any AP that was not seen (value of 0) is set to -100
    Data(Data==0)=-100;
    
    % Stores the number of different locations in the training data
    numLoc=size(cartesianLabels,1);
    
    % Stores the number of data points, for each location
    dataPerLoc=size(Data,1)/numLoc;
    
    for n=1:50
        n
        for i=5:5:95
            % Setup our training data array
            trainingSamplesPerClass = floor(dataPerLoc*i/100);
            classificationSamplesPerClass = ceil(dataPerLoc*(100-i)/100);
            trainingData = zeros(numLoc*trainingSamplesPerClass, size(Data,2));
            classificationData = zeros(numLoc*classificationSamplesPerClass, size(Data,2));
            trainingLabels = zeros(numLoc*trainingSamplesPerClass, 1);
            classificationLabels=zeros(numLoc*classificationSamplesPerClass, 1);

            % Loop through all the test cases selecting random samples
            for j=1:numLoc
                % Set up training samples
                r=randperm(dataPerLoc)+(j-1)*dataPerLoc;
                trainingData((((j-1)*trainingSamplesPerClass)+1):(j*trainingSamplesPerClass),:) = Data(r(1:trainingSamplesPerClass),:);  
                classificationData((((j-1)*classificationSamplesPerClass)+1):(j*classificationSamplesPerClass),:) = Data(r(trainingSamplesPerClass+1:size(r,2)),:);  
                trainingLabels((((j-1)*trainingSamplesPerClass)+1):(j*trainingSamplesPerClass),:) = j;  
                classificationLabels((((j-1)*classificationSamplesPerClass)+1):(j*classificationSamplesPerClass),:) = j;
            end

            if(onlyUCM)
                filename = strcat('GorkemUCM/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            else
                filename = strcat('GorkemAll/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            end
            save(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels', 'MacIndices');
        end  
    end
end
