function SVMoneVall(DatasetFolder, nStart, nEnd)
    if(nargin<1)
        DatasetFolder='Kavraki'; % Use Ben dataset by default
    end
    
    if(strcmp(DatasetFolder,'GorkemUCM'))
        % Train the Logistic Regression with One against All methodology
        SVMoneVall_Train_GorkemUCM(DatasetFolder, nStart, nEnd);

        % Classify the data
        SVMoneVall_Classify_GorkemUCM(DatasetFolder);
    elseif(strcmp(DatasetFolder,'Kavraki'))
        % Train the Logistic Regression with One against All methodology
        SVMoneVall_Train_Kavraki(DatasetFolder);

        % Classify the data
        SVMoneVall_Classify_Kavraki(DatasetFolder); 
    end
end

function SVMoneVall_Classify_GorkemUCM(DatasetFolder)
    for k=1:19
        j=k*5;
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');
            filename = strcat(DatasetFolder, '/SVMoneVall/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'xsup','w','b','nbsv','kernel','kerneloption');

            %yfit = svmmultivaloneagainstone(classificationData,xsup,w,b,nbsv,kernel,kerneloption);
            tic;
            yfit = svmmultival(classificationData,xsup,w,b,nbsv,kernel,kerneloption);
            Timing(n,1)=toc;
            
            D=0.3048*sqrt(sum((cartesianLabels(yfit,:)-cartesianLabels(classificationLabels,:)).^2,2));
            Distance(n,:)=D';

            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/size(classificationLabels,1);
            end          
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(k).X=gdiErrorPlotX;
        errorPlot(k).Y=mean(gdiErrorPlotY);
        errorPlot(k).Distance=Distance;
        errorPlot(k).Timing=Timing;
        
        %        figure   
        % If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         en
    end
   
    % Save the data to plot later on...
    %filename = strcat(DatasetFolder, '/SVMoneVall/Results.mat');
    %save(filename, 'errorPlot');
end

function SVMoneVall_Train_GorkemUCM(DatasetFolder, nStart, nEnd)
    for n=nStart:nEnd
        for i=5:5:95
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData', 'trainingLabels', 'classificationData', 'classificationLabels', 'cartesianLabels');

            numLoc=size(cartesianLabels,1);
            
            %-----------------------------------------------------
            % Learning and Learning Parameters
            %-----------------------------------------------------
            c = 1000;
            lambda = 1e-7;
            kerneloption=30;
            %kernel='poly';
            kernel='gaussian';
            %kernel='polyhomog';
            verbose = 0;
            nbclass=numLoc;

            tic;
            % Train our classifier
            %[xsup,w,b,nbsv,classifier]=svmmulticlassoneagainstone(trainingData,trainingLabels,nbclass,c,lambda,kernel,kerneloption,verbose);
            [xsup,w,b,nbsv,classifier]=svmmulticlassoneagainstall(trainingData,trainingLabels,nbclass,c,lambda,kernel,kerneloption,verbose);
            trainingTime=toc;
            
            filename = strcat(DatasetFolder, '/SVMoneVall/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'xsup','w','b','nbsv','kernel','kerneloption', 'trainingTime');
        end  
    end
end

function SVMoneVall_Classify_Kavraki(DatasetFolder)
    for j=1:19
        gdiErrorPlotX=[];
        gdiErrorPlotY=[];
        Distance=[];
        Timing=[];
        for n=1:50
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');
            filename = strcat(DatasetFolder, '/SVMoneVall/Trained_', num2str(j,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'xsup_F1','w_F1','b_F1','nbsv_F1', 'xsup_F2','w_F2','b_F2','nbsv_F2', 'xsup_F3','w_F3','b_F3','nbsv_F3','kernel','kerneloption', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');

            %yfit = svmmultivaloneagainstone(classificationData,xsup,w,b,nbsv,kernel,kerneloption);
            tic;
            yfit_F1 = svmmultival(classificationData_F1,xsup_F1,w_F1,b_F1,nbsv_F1,kernel,kerneloption);
            Timing(n,1)=toc;
            D_F1=sqrt(sum((cartesianLabels_F1(yfit_F1,:)-cartesianLabels_F1(classificationLabels_F1,:)).^2,2))/16.68;
            
            tic;
            yfit_F2 = svmmultival(classificationData_F2,xsup_F2,w_F2,b_F2,nbsv_F2,kernel,kerneloption);
            Timing(n,2)=toc;
            D_F2=sqrt(sum((cartesianLabels_F2(yfit_F2,:)-cartesianLabels_F2(classificationLabels_F2,:)).^2,2))/16.68;
            
            tic;
            yfit_F3 = svmmultival(classificationData_F3,xsup_F3,w_F3,b_F3,nbsv_F3,kernel,kerneloption);
            Timing(n,3)=toc;
            D_F3=sqrt(sum((cartesianLabels_F3(yfit_F3,:)-cartesianLabels_F3(classificationLabels_F3,:)).^2,2))/16.68;
            
            D=[D_F1;D_F2;D_F3];
            Distance(n,:)=D';

            % Get error rate for every 25cm
            var=size(gdiErrorPlotY,1)+1;
            for i=1:ceil(max(D))*4+1
                %gdiErrorPlotX(var,i)=0.25*(i-1);
                gdiErrorPlotY(var,i)=sum(D<=0.25*(i-1))/(size(classificationLabels_F1,1)+size(classificationLabels_F2,1)+size(classificationLabels_F3,1));
            end          
        end
        
        % Clean up array to remove extra zeros
        for i=1:size(gdiErrorPlotY,1)
            indices = find(gdiErrorPlotY(i,:)==1);
            gdiErrorPlotY(i,indices(size(indices,2)):size(gdiErrorPlotY,2))=1;
        end

        %hold on
        gdiErrorPlotX = 0:0.25:(size(gdiErrorPlotY,2)-1)/4;
        
        errorPlot(j).X=gdiErrorPlotX;
        errorPlot(j).Y=mean(gdiErrorPlotY);
        errorPlot(j).Distance=Distance;
        errorPlot(j).Timing=Timing;
        
%         figure   
%         %If we only have one dataset
%         if(size(gdiErrorPlotY,1)==1)
%             plot(gdiErrorPlotX, gdiErrorPlotY, 'b');
%         else
%             plot(gdiErrorPlotX, mean(gdiErrorPlotY), 'b');
%         end
    end
   
    % Save the data to plot later on...
    filename = strcat(DatasetFolder, '/SVMoneVall/Results.mat');
    save(filename, 'errorPlot');
end

function SVMoneVall_Train_Kavraki(DatasetFolder)
    for n=1:50
        for i=1:19
            n
            filename = strcat(DatasetFolder, '/TrainingRandomized/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            load(filename, 'trainingData_F1', 'trainingLabels_F1', 'classificationData_F1', 'classificationLabels_F1', 'cartesianLabels_F1','trainingData_F2', 'trainingLabels_F2', 'classificationData_F2', 'classificationLabels_F2', 'cartesianLabels_F2','trainingData_F3', 'trainingLabels_F3', 'classificationData_F3', 'classificationLabels_F3', 'cartesianLabels_F3', 'MacIndices');
            
            %-----------------------------------------------------
            % Learning and Learning Parameters
            %-----------------------------------------------------
            c = 1000;
            lambda = 1e-7;
            kerneloption=30;
            kernel='gaussian';
            verbose = 0;
            nbclass_F1=size(cartesianLabels_F1,1);
            nbclass_F2=size(cartesianLabels_F2,1);
            nbclass_F3=size(cartesianLabels_F3,1);

            tic;
            [xsup_F1,w_F1,b_F1,nbsv_F1,classifier_F1]=svmmulticlassoneagainstall(trainingData_F1,trainingLabels_F1,nbclass_F1,c,lambda,kernel,kerneloption,verbose);
            trainingTime_F1=toc;
            
            tic;
            [xsup_F2,w_F2,b_F2,nbsv_F2,classifier_F2]=svmmulticlassoneagainstall(trainingData_F2,trainingLabels_F2,nbclass_F2,c,lambda,kernel,kerneloption,verbose);
            trainingTime_F2=toc;
            
            tic;
            [xsup_F3,w_F3,b_F3,nbsv_F3,classifier_F3]=svmmulticlassoneagainstall(trainingData_F3,trainingLabels_F3,nbclass_F3,c,lambda,kernel,kerneloption,verbose);
            trainingTime_F3=toc;
            
            filename = strcat(DatasetFolder, '/SVMoneVall/Trained_', num2str(i,'%02d'), '_', num2str(n,'%02d'), '.mat');
            save(filename, 'xsup_F1','w_F1','b_F1','nbsv_F1', 'xsup_F2','w_F2','b_F2','nbsv_F2', 'xsup_F3','w_F3','b_F3','nbsv_F3','kernel','kerneloption', 'trainingTime_F1', 'trainingTime_F2', 'trainingTime_F3');
        end  
    end
end

